// test if the parser works correctly

// Import the parse function and AST module
const { parse } = require("../../src/utils/parser");
const ast = require("../../src/utils/ast");

describe('Parser', () => {

    // expression test
    test('should parse True expression', () => {
        const input = "true"; // Input string
        const parsedResult = parse(input); // Parse the input
        const expectedResult = ast.True();
        expect(parsedResult).toEqual(expectedResult); // Assert parsed result matches expected result
    });

    test('should parse False expression', () => {
        const input = "false"; // Input string
        const parsedResult = parse(input); // Parse the input
        const expectedResult = ast.False();
        expect(parsedResult).toEqual(expectedResult); // Assert parsed result matches expected result
    });

    test('should parse Symbol expression', () => {
        const input = "A"; // Input string
        const parsedResult = parse(input); // Parse the input
        const expectedResult = ast.Symbol("A",null,null);
        expect(parsedResult).toEqual(expectedResult); // Assert parsed result matches expected result
    });

    test('should parse not expression', () => {
        const input = "!A"; // Input string
        const parsedResult = parse(input); // Parse the input
        const expectedResult = ast.Not("!",ast.Symbol("A",null,null),null);
        expect(parsedResult).toEqual(expectedResult); // Assert parsed result matches expected result
    });

    test('should parse and expression', () => {
        const input = "A & B"; // Input string
        const parsedResult = parse(input); // Parse the input
        const expectedResult = ast.And("&",ast.Symbol("A",null,null),ast.Symbol("B",null,null));
        expect(parsedResult).toEqual(expectedResult); // Assert parsed result matches expected result
    });

    test('should parse or expression', () => {
        const input = "A || B"; // Input string
        const parsedResult = parse(input); // Parse the input
        const expectedResult = ast.Or("||",ast.Symbol("A",null,null),ast.Symbol("B",null,null));
        expect(parsedResult).toEqual(expectedResult); // Assert parsed result matches expected result
    });

    test('should parse imply expression', () => {
        const input = "A -> B"; // Input string
        const parsedResult = parse(input); // Parse the input
        const expectedResult = ast.Imply("->",ast.Symbol("A",null,null),ast.Symbol("B",null,null));
        expect(parsedResult).toEqual(expectedResult); // Assert parsed result matches expected result
    });

    // sequent test
    test('should parse True', () => {
        const input = "|- true"; // Input string
        const parsedResult = parse(input); // Parse the input
        const expectedResult = [[], [ast.True()]];
        expect(parsedResult).toEqual(expectedResult); // Assert parsed result matches expected result
    });

    test('should parse False', () => {
        const input = "|- false"; // Input string
        const parsedResult = parse(input); // Parse the input
        const expectedResult = [[], [ast.False()]];
        expect(parsedResult).toEqual(expectedResult); // Assert parsed result matches expected result
    });

    test('should parse symbol', () => {
        const input = "|- A"; // Input string
        const parsedResult = parse(input); // Parse the input
        const expectedResult = [[], [ast.Symbol("A",null,null)]];
        expect(parsedResult).toEqual(expectedResult); // Assert parsed result matches expected result
    });

    test('should parse symbol with brackets', () => {
        const input = "|- (A)"; // Input string
        const parsedResult = parse(input); // Parse the input
        const expectedResult = [[], [ast.Symbol("A",null,null)]];
        expect(parsedResult).toEqual(expectedResult); // Assert parsed result matches expected result
    });

    test('should parse simple not expression', () => {
        const input = "|- ! A"; // Input string
        const parsedResult = parse(input); // Parse the input
        const expectedResult = [[], // Expected result AST
            [ast.Not("!",ast.Symbol("A",null,null), null)]];
        expect(parsedResult).toEqual(expectedResult); // Assert parsed result matches expected result
    });

    test('should parse simple and expression', () => {
        const input = "|- A & B"; // Input string
        const parsedResult = parse(input); // Parse the input
        const expectedResult = [[], // Expected result AST
            [ast.And("&",ast.Symbol("A",null,null), ast.Symbol("B",null,null))]];
        expect(parsedResult).toEqual(expectedResult); // Assert parsed result matches expected result
    });

    test('should parse simple or expression', () => {
        const input = "|- A || B"; // Input string
        const parsedResult = parse(input); // Parse the input
        const expectedResult = [[], // Expected result AST
            [ast.Or("||",ast.Symbol("A",null,null), ast.Symbol("B",null,null))]];
        expect(parsedResult).toEqual(expectedResult); // Assert parsed result matches expected result
    });


    test('should parse simple imply expression', () => {
        const input = "|- A -> B"; // Input string
        const parsedResult = parse(input); // Parse the input
        const expectedResult = [[], // Expected result AST
            [ast.Imply("->",ast.Symbol("A",null,null), ast.Symbol("B",null,null))]];
        expect(parsedResult).toEqual(expectedResult); // Assert parsed result matches expected result
    });

    test('should parse symbol in given and conclusion', () => {
        const input = "A |- A"; // Input string
        const parsedResult = parse(input); // Parse the input
        const expectedResult = [[ast.Symbol("A",null,null)], // Expected result AST
            [ast.Symbol("A",null,null)]];
        expect(parsedResult).toEqual(expectedResult); // Assert parsed result matches expected result
    });


    test('should parse simple sequent with and', () => {
        const input = "A & B |- C"; // Input string
        const parsedResult = parse(input); // Parse the input
        const expectedResult = [
            [ast.And("&",ast.Symbol("A",null,null), ast.Symbol("B",null,null))], // Expected result AST
            [ast.Symbol("C",null,null)]
        ];
        expect(parsedResult).toEqual(expectedResult); // Assert parsed result matches expected result
    });

    test('should parse simple sequent with or', () => {
        const input = "A || B |- C"; // Input string
        const parsedResult = parse(input); // Parse the input
        const expectedResult = [
            [ast.Or("||",ast.Symbol("A",null,null), ast.Symbol("B",null,null))], // Expected result AST
            [ast.Symbol("C",null,null)]
        ];
        expect(parsedResult).toEqual(expectedResult); // Assert parsed result matches expected result
    });

    test('should parse sequent with multiple givens(&)', () => {
        const input = "A, B & C |- D"; // Input string
        const parsedResult = parse(input); // Parse the input
        const expectedResult = [
            [ast.Symbol("A",null,null), 
            ast.And("&",ast.Symbol("B",null,null), ast.Symbol("C",null,null))], // Expected result AST
            [ast.Symbol("D",null,null)]
        ];
        expect(parsedResult).toEqual(expectedResult); // Assert parsed result matches expected result
    });

    test('should parse sequent with multiple givens(||)', () => {
        const input = "A, B || C |- D"; // Input string
        const parsedResult = parse(input); // Parse the input
        const expectedResult = [
            [ast.Symbol("A",null,null), 
            ast.Or("||",ast.Symbol("B",null,null), ast.Symbol("C",null,null))], // Expected result AST
            [ast.Symbol("D",null,null)]
        ];
        expect(parsedResult).toEqual(expectedResult); // Assert parsed result matches expected result
    });

    test('should parse sequent with multiple conclusions', () => {
        const input = "A, B || C |- D, C"; // Input string
        const parsedResult = parse(input); // Parse the input
        const expectedResult = [
            [ast.Symbol("A",null,null), 
            ast.Or("||",ast.Symbol("B",null,null), ast.Symbol("C",null,null))], // Expected result AST
            [ast.Symbol("D",null,null), ast.Symbol("C",null,null)]
        ];
        expect(parsedResult).toEqual(expectedResult); // Assert parsed result matches expected result
    });

    test('should parse sequent with multiple givens(||) and & in condition', () => {
        const input = "A, B || C |- D & C"; // Input string
        const parsedResult = parse(input); // Parse the input
        const expectedResult = [
            [ast.Symbol("A",null,null), 
            ast.Or("||",ast.Symbol("B",null,null), ast.Symbol("C",null,null))], // Expected result AST
            [ast.And("&",ast.Symbol("D",null,null), ast.Symbol("C",null,null))]
        ];
        expect(parsedResult).toEqual(expectedResult); // Assert parsed result matches expected result
    });

    // associativity

    // right associate
    test('should parse sequent with -> in condition', () => {
        const input = "|- A -> B -> C"; // Input string
        const parsedResult = parse(input); // Parse the input
        const expectedResult = [
            [],
            [ast.Imply("->",ast.Symbol("A", null, null), 
            ast.Imply("->",ast.Symbol("B", null, null), ast.Symbol("C", null, null)))]
        ];
        expect(parsedResult).toEqual(expectedResult); // Assert parsed result matches expected result
    });

    // left associate
    test('should parse sequent with & in condition', () => {
        const input = "|- A & B & C"; // Input string
        const parsedResult = parse(input); // Parse the input
        const expectedResult = [
            [],
            [ast.And("&",ast.And("&",ast.Symbol("A", null, null), ast.Symbol("B", null, null)),
            ast.Symbol("C", null, null))]
        ];
        expect(parsedResult).toEqual(expectedResult); // Assert parsed result matches expected result
    });

    // left associate
    test('should parse sequent with || in condition', () => {
        const input = "|- A || B || C"; // Input string
        const parsedResult = parse(input); // Parse the input
        const expectedResult = [
            [],
            [ast.Or("||",ast.Or("||",ast.Symbol("A", null, null), ast.Symbol("B", null, null)),
            ast.Symbol("C", null, null))]
        ];
        expect(parsedResult).toEqual(expectedResult); // Assert parsed result matches expected result
    });

    // precedence test
    test('should parse sequent with not and & in condition', () => {
        const input = "|- ! D & C"; // Input string
        const parsedResult = parse(input); // Parse the input
        const expectedResult = [
            [],
            [ast.And("&",ast.Not("!",ast.Symbol("D", null, null), null), ast.Symbol("C",null,null))]
        ];
        expect(parsedResult).toEqual(expectedResult); // Assert parsed result matches expected result
    });

    test('should parse sequent with not and & in condition', () => {
        const input = "|- ! (D & C)"; // Input string
        const parsedResult = parse(input); // Parse the input
        const expectedResult = [
            [],
            [ast.Not("!",ast.And("&",ast.Symbol("D", null, null), 
            ast.Symbol("C",null,null)), null)]
        ];
        expect(parsedResult).toEqual(expectedResult); // Assert parsed result matches expected result
    });

    test('should parse sequent with || and & in condition', () => {
        const input = "|- A || !D & C"; // Input string
        const parsedResult = parse(input); // Parse the input
        const expectedResult = [
            [],
            [ast.Or("||",ast.Symbol("A", null, null), 
            ast.And("&",ast.Not("!", ast.Symbol("D", null, null), null), 
            ast.Symbol("C", null, null)))]
        ];
        expect(parsedResult).toEqual(expectedResult); // Assert parsed result matches expected result
    });

    test('should parse sequent with -> and brackets in condition', () => {
        const input = "|- (A -> !D) -> C"; // Input string
        const parsedResult = parse(input); // Parse the input
        const expectedResult = [
            [],
            [ast.Imply("->", ast.Imply("->",ast.Symbol("A", null, null), 
            ast.Not("!", ast.Symbol("D", null, null), null)), ast.Symbol("C", null, null),)]
        ];
        expect(parsedResult).toEqual(expectedResult); // Assert parsed result matches expected result
    });

    test('should parse sequent with -> and brackets in condition', () => {
        const input = "|- (A->B) || !C & D"; // Input string
        const parsedResult = parse(input); // Parse the input
        const expectedResult = [
            [],
            [ast.Or("||", ast.Imply("->", ast.Symbol("A", null, null), ast.Symbol("B", null, null)), 
            ast.And("&", ast.Not("!", ast.Symbol("C", null, null), null), ast.Symbol("D", null, null)))]
        ];
        expect(parsedResult).toEqual(expectedResult); // Assert parsed result matches expected result
    });






});
